--
-- coup
--
-- Copyright (c) 2014-2016, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

local function addFrames(animation, img, frames, frameIndices)
  for i, fi in ipairs(frameIndices) do
    local f = frames[fi]
    local fr = f.frame
    local ss = f.spriteSourceSize
    animation:addFrame(img, fr.x, fr.y, fr.w, fr.h, ss.x, ss.y, f.duration / 1000.0)
  end
end

local function loadAsepriteFile(filename, repository, loadImage)
  local parts = {}
  for m in string.gmatch(filename, '[^/]+') do
    parts[#parts + 1] = m
  end
  local data = love.filesystem.read(filename)
  local json = coup.json.decode(data)

  local meta = json.meta
  local frames = json.frames
  parts[#parts] = meta.image

  local img = loadImage(table.concat(parts, '/'))
  
  for i, tag in ipairs(meta.frameTags) do
    local anim = repository:addAnimation(tag.name)
    
    if tag.direction == "forward" or tag.direction == "pingpong" then
      addFrames(anim, img, frames, coup.tables.range(tag.from+1, tag.to+1))
    elseif tag.direction == "reverse" then
      addFrames(anim, img, frames, coup.tables.range(tag.to+1, tag.from+1, -1))
    end

    -- Add reverse direction for pinpong
    if tag.direction == "pingpong" then
      addFrames(anim, img, frames, coup.tables.range(tag.to, tag.from+2, -1))
    end

  end

end

return { loadAsepriteFile = loadAsepriteFile }
