--
-- coup
--
-- Copyright (c) 2014-2016, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

local BASE = (...):match("(.-)[^%.]+$")
local tables = require(BASE .. ".tables")


-- Construct an instance of the given class by calling its new function
local construct = function(class, ...)
  local new = tables.merge(class)
  class.new(new, ...)
  return new
end


-- Recursive inheritance check:
-- Returns true if a inherits from b (directly or indirectly),
-- false otherwise
local inherits = function(a, b)
  if b == a then
    return true
  end

  if a.super then
    return a.super:inherits(b)
  end

  return false
end


-- Baseclass of all Objects
local Object = {
  new = function() end,
  inherits = inherits
}


-- Allow the creation of new classes.
-- If b1 is an existing class, then functions in b2 will overwrite functions in b1,
-- otherwise all functions in b1 will be in the new class
local class = function(b1, b2)
  -- Use Object as baseclass by default
  local base = b2 and b1 or Object
  local ext  = b2 and b2 or b1

  -- Merge baseclass's and new methods -> no daisy chaining of __index
  local predef = tables.merge(base, ext)

  -- Store super class and own class
  predef.super = base
  predef.class = predef

  return setmetatable(predef, {__call = construct})
end


return class
