--
-- coup
--
-- Copyright (c) 2014-2016, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

local ComponentStore = {}
ComponentStore.__index = ComponentStore


function ComponentStore:onRemove(func)
  self.removeFuncs[#self.removeFuncs+1] = func
end


function ComponentStore:onMake(func)
  self.makeFuncs[#self.makeFuncs+1] = func
end


function ComponentStore:onAttach(func)
  self.attachFuncs[#self.attachFuncs+1] = func
end


function ComponentStore:removeOnAttach(func)
  local i = ctables.array_find(self.attachFuncs, func)

  if i then
    table.remove(self.attachFuncs, i)
  end
end


function ComponentStore:removeOnMake(func)
  local i = ctables.array_find(self.makeFuncs, func)

  if i then
    table.remove(self.makeFuncs, i)
  end
end


function ComponentStore:removeOnRemove(func)
  local i = ctables.array_find(self.removeFuncs, func)

  if i then
    table.remove(self.removeFuncs, i)
  end
end


function ComponentStore:attached(comp)
  for i, f in ipairs(self.attachFuncs) do
    f(comp)
  end
end


function ComponentStore:remove(comp)
  if not self.components[comp] then
    error("Trying to remove non-existent component")
  end

  for i, f in ipairs(self.removeFuncs) do
    f(comp)
  end

  self.components[comp] = nil
  self.count = self.count - 1
end


function ComponentStore:make(...)
  self.count = self.count + 1
  local comp = self.makeFunc(...)
  comp.store = self
  self.components[comp] = comp

  for i, f in ipairs(self.makeFuncs) do
    f(comp)
  end
  return comp
end


local function newComponentStore(make)
  return setmetatable({
    makeFunc = make or function() return {} end,
    components = {},
    count = 0,
    removeFuncs = {},
    makeFuncs = {},
    attachFuncs = {}
  }, ComponentStore)
end


return newComponentStore
