--
-- coup
--
-- Copyright (c) 2014-2016, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

local Entity = {}
Entity.__index = Entity


local function newEntity(world, name)
  return setmetatable({
    world = world,
    components = {},
    name = name
  }, Entity)
end


function Entity:makeComponent(...)
  local comp = self.world:makeComponent(...)
  self:addComponent(comp)
  return comp
end


function Entity:addComponent(component)
  assert(component.type, "Component must have a field [type]")
  assert(self.components[component.type] == nil, "Entity can only have one component of each type")
  assert(component.entity == nil, "Component cannot only be attached to one entity")

  self.components[component.type] = component
  component.entity = self

  component.store:attached(component)
end


function Entity:removeComponent(typeOrInstance)
  if type(typeOrInstance) == "table" then
    self:removeComponent(typeOrInstance.type)
    return
  end

  local component = self.components[typeOrInstance]

  component.store:remove(component)

  component.entity = nil
  self.components[typeOrInstance] = nil
end


function Entity:getComponent(type)
  return self.components[type]
end


-- TODO write test :)
function Entity:getComponents(...)
  local res = {}
  local cs = self.components
  for i, compType in ipairs({...}) do
    local c = cs[compType]
    if not c then
      return nil
    end
    res[#res+1] = c
  end
  return unpack(res)
end


function Entity:destroy()
  for k in pairs(self.components) do
    self:removeComponent(k)
  end
end


return { Entity = newEntity }
