--
-- coup
--
-- Copyright (c) 2014-2016, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

local function iterLineDDA(x1, y1, x2, y2)
  local mapX = math.floor(x1)
  local mapY = math.floor(y1)
  local dx = x2 - x1
  local dy = y2 - y1

  local sq = coup.smath.sq
  local sqrt = math.sqrt
  local floor = math.floor

  local deltaDistX = sqrt(1 + sq(dy) / sq(dx))
  local deltaDistY = sqrt(1 + sq(dx) / sq(dy))
  local stepX
  local stepY
  local hit = false
  local side
  local sideDistX, sideDistY

  if dx < 0 then
    stepX = -1
    sideDistX = (x1 - mapX) * deltaDistX
  else
    stepX = 1
    sideDistX = (mapX + 1.0 - x1) * deltaDistX
  end

  if dy < 0 then
    stepY = -1
    sideDistY = (y1 - mapY) * deltaDistY
  else
    stepY = 1
    sideDistY = (mapY + 1.0 - y1) * deltaDistY
  end

  return coroutine.wrap(function()
    coroutine.yield(mapX, mapY)

    local mapLastX = floor(x2)
    local mapLastY = floor(y2)
    while (mapLastY ~= mapY or mapLastX ~= mapX) and not hit do
      if sideDistX < sideDistY then
        sideDistX = sideDistX + deltaDistX
        mapX = mapX + stepX
        side = 0
      else
        sideDistY = sideDistY + deltaDistY
        mapY = mapY + stepY
        side = 1
      end

      coroutine.yield(mapX, mapY)
    end
  end)

end


local function iterLine(x1, y1, x2, y2)
  if x1 == x2 and y1 == y2 then
    return function()
      return x1, y1
    end
  end
  return coroutine.wrap(function()
    local minstep = 1 / math.max(math.abs(x2 - x1), math.abs(y2 - y1))
    local stepx = (x2 - x1) * minstep
    local stepy = (y2 - y1) * minstep
    local lastx, lasty
    local round = coup.smath.round
    coroutine.yield(x1, y1)
    while x1 ~= x2 do
      x1 = x1 + stepx
      y1 = y1 + stepy
      local x, y = round(x1), round(y1)
      if x ~= lastx or y ~= lasty then
        coroutine.yield(x, y)
        lastx, lasty = x, y
      end
    end
  end)
end


return {
  iterLine = iterLine,
  iterLineDDA = iterLineDDA
}
