--
-- coup
--
-- Copyright (c) 2014-2016, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

local smath = {}

-- return x cubed
function smath.cu(x)
  return x*x*x
end


-- return x squared
function smath.sq(x)
  return x*x
end


-- linearly interpolate between a and b
local function lerp(a, b, alpha)
  return (1 - alpha) * a + alpha * b
end
smath.lerp = lerp


local pi2 = math.pi * .5
math.pi2 = pi2

local interpolationModes = {
  linear      = function(t) return t end,
  sine_inout  = function(t) return (1 - math.cos(t * math.pi)) * 0.5 end,
  sine_in     = function(t) return  1 - math.cos(t * pi2)            end,
  sine_out    = function(t) return      math.cos((1-t) * pi2)        end,
}

local interpolationFuncs = {}
for k, v in pairs(interpolationModes) do
  interpolationFuncs[k] = function(a, b, alpha) return smath.lerp(a, b, v(alpha)) end
end


-- returns an interpolation function using the given easing mode
function smath.interpolate_func(mode)
  return interpolationFuncs[mode]
end


-- interpolate between a and b with different easing modes
function smath.interpolate(a, b, alpha, mode)
  return interpolationFuncs[mode](a, b, alpha)
end


-- check whether a is between b and c. the order of b and c does not matter.
function smath.between(a, b, c)
  return a >= math.min(b, c) and a <= math.max(b, c)
end


-- return sign of a: -1 if a < 0, 0 if a == 0 and 1 if a > 0
function smath.sign(a)
  if a < 0 then
    return -1
  elseif a > 0 then
    return 1
  else
    return 0
  end
end


function smath.round(a, step)
  if step then
    return smath.round(a / step) * step
  end

  if a >= 0 then
    return math.floor(a + 0.5)
  end

  return math.ceil(a - 0.5)
end


return smath
