--
-- coup
--
-- Copyright (c) 2014-2016, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

local AABBUpdater = {}
AABBUpdater.__index = AABBUpdater


function AABBUpdater_updateEntity(e)
  local bb = e:getComponent('BoundingBox')
  local tr = e:getComponent('Transform')

  local r = tr.rot
  local p1 = bb.p1
  local p2 = bb.p2
  local p = tr.pos

  local vec = coup.vector.Vector
  local s = tr.scale

  local vs = {
    (p1:rotatedZ(r)+p)*s,
    (vec(p1.x, p2.y):rotatedZ(r)+p)*s,
    (p2:rotatedZ(r)+p)*s,
    (vec(p2.x, p1.y):rotatedZ(r)+p)*s
  }

  local minX = math.huge
  local minY = math.huge
  local maxX = -math.huge
  local maxY = -math.huge

  for i = 1, 4 do
    minX = math.min(vs[i].x, minX)
    maxX = math.max(vs[i].x, maxX)
    minY = math.min(vs[i].y, minY)
    maxY = math.max(vs[i].y, maxY)
  end

  local aabb = e:getComponent('AxisAlignedBoundingBox')

  if not aabb then
    e:makeComponent('AxisAlignedBoundingBox', minX, minY, maxX, maxY)
    if not e:getComponent('AABBChanged') then
      e:makeComponent('AABBChanged')
    end
    return
  end

  local tl = aabb.topLeft
  local br = aabb.bottomRight

  if minX ~= tl.x or minY ~= tl.y or maxX ~= br.x or maxY ~= br.y then
    tl:set(minX, minY)
    br:set(maxX, maxY)
    if not e:getComponent('AABBChanged') then
      e:makeComponent('AABBChanged')
    end
  end

end


local function removeOldAABBChanged(e)
  e:removeComponent("AABBChanged")
end


function AABBUpdater:update()
  for i, e in ipairs(self.deadEntities) do
    e:removeComponent('AxisAlignedBoundingBox')
    self.deadEntities[i] = nil
  end

  self.world:foreachEntityWithComponents(removeOldAABBChanged, {"AABBChanged"})

  self.world:foreachEntityWithComponents(AABBChanged_updateEntity, {"BoundingBox", "Transform"})
end


local function newAABBUpdater(world)
  local self = {
    world = world,
    deadEntities = {}
  }
  world.componentTypes.BoundingBox:onRemove(function(c) self.deadEntities[#self.deadEntities+1] = c.entity end)

  return setmetatable(self, AABBUpdater)
end


return newAABBUpdater
