--
-- coup
--
-- Copyright (c) 2014-2016, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

local AutoHasher = {}
AutoHasher.__index = AutoHasher


function AutoHasher:update()
  local dE = self.deadEntities
  for i, e in ipairs(dE) do
    self.hashTable:remove(e)
    dE[i] = nil
  end

  self.world:foreachEntityWithComponents(function(e) 
    local aabb = e.components.AxisAlignedBoundingBox
    local tl = aabb.topLeft
    local br = aabb.bottomRight
    self.hashTable:insertAABB(e, tl.x, tl.y, br.x, br.y)
  end, {"AABBChanged", unpack(self.requiredComps)})

  local nC = self.newComponents
  for i, c in ipairs(nC) do
    local e = c.entity
    local aabb = e:getComponents('AxisAlignedBoundingBox', unpack(self.requiredComps))
    if aabb then
      local tl = aabb.topLeft
      local br = aabb.bottomRight
      self.hashTable:insertAABB(e, tl.x, tl.y, br.x, br.y)
      nC[i] = nil
    end
  end
end


function AutoHasher:newAABBComponent(comp)
  local nC = self.newComponents
  nC[#nC+1] = comp
end


function AutoHasher:removeAABBComponent(comp)
  local dE = self.deadEntities
  dE[#dE+1] = comp.entity
end


function AutoHasher:enable()
  self.world.componentTypes.AxisAlignedBoundingBox:onAttach(function(c) self:newAABBComponent(c) end)
  self.world.componentTypes.AxisAlignedBoundingBox:onRemove(function(c) self:removeAABBComponent(c) end)
end


local function newAutoHasher(spatialHashtable, world, ...)

  local hasher = setmetatable({
    hashTable = spatialHashtable,
    world = world,
    newComponents = {},
    deadEntities = {},
    requiredComps = {...}
  }, AutoHasher)


  hasher:enable()

  return hasher
end


return newAutoHasher
