--
-- coup
--
-- Copyright (c) 2014-2016, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

local SpriteRenderer = {}
SpriteRenderer.__index = SpriteRenderer

local function collectSprites(tl, br, hash)
  local layers = {}

  for e in hash:iterEntitiesAABB(tl.x, tl.y, br.x, br.y) do
    local s, t = e:getComponents('Sprite', 'Transform')
    if s then
      local f = s.sprite:getCurrentFrame()
      if f then
        local layer = layers[s.layer]
        if not layer then
          layer = {}
          layers[s.layer] = layer
        end

        layer[#layer+1] = {f, t}
      end
    end
  end

  return layers
end


local function compareLayers(a, b)
  return a[1] < b[1] 
end

local function sortSpriteLayers(layers)
  local sortLayers = {}
  for k, v in pairs(layers) do
    sortLayers[#sortLayers+1] = {k, v}
  end

  table.sort(sortLayers, compareLayers)

  return sortLayers
end


local function prepareSpriteBatches(sbs, entries)
  for k, e in ipairs(entries) do
    local f = e[1]

    local sb = sbs[f.img]
    if sb == nil then
      sb = love.graphics.newSpriteBatch(f.img, 5000, "stream")
      sbs[f.img] = sb
    end
    local t = e[2]
    local p = t.pos
    local r = t.rot
    local s = t.scale
    local o = f.origin
    sb:add(f.quad, math.floor(p.x), math.floor(p.y), r, s.x, s.y, o.x, o.y)
  end
end


function SpriteRenderer:renderVisible(camera)
  local bounds = camera:getComponent("AxisAlignedBoundingBox")
  local tl = bounds.topLeft
  local br = bounds.bottomRight

  local layers = collectSprites(tl, br, self.spatialHash)
  local sortLayers = sortSpriteLayers(layers)

  local cam_size, screen_offset, offset = coup.camera.cameraSettings(tl, br)

  love.graphics.setColor(255,255,255,255)
  local sbs = self.spriteBatches
  for i, l in ipairs(sortLayers) do
    prepareSpriteBatches(sbs, l[2])

    for j, s in pairs(self.spriteBatches) do
      love.graphics.draw(s, math.floor(offset.x), math.floor(offset.y))
      s:clear()
    end
  end

end


local function newSpriteRenderer(spatialHash)
  return setmetatable({
    spatialHash = spatialHash,
    spriteBatches = {}
  }, SpriteRenderer)
end

return newSpriteRenderer
