--
-- coup
--
-- Copyright (c) 2014-2016, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--


local function loadTiledMap(filename, imageLoader)
  local mapfile = love.filesystem.load(filename)()

  local map = coup.tilemap.Tilemap(mapfile.tilewidth, mapfile.tileheight, mapfile.width, mapfile.height)

  for i, t in ipairs(mapfile.tilesets) do
    local img = imageLoader(t.image)
    map:add_tileset(coup.tileset.Tileset(img, t.firstgid, t.tilewidth, t.tileheight))
  end

  local objectgroups = {}
  local imagelayers = {}
  for i, l in ipairs(mapfile.layers) do
    if l.type == "tilelayer" then
      map:add_layer(l.name, l.data)
    elseif l.type == "objectgroup" then
      objectgroups[l.name] = l
    elseif l.type == "imagelayer" then
      imagelayers[l.name] = l
    end
  end

  return map, objectgroups, imagelayers
end

return { loadTiledMapFile = loadTiledMap }
