--
-- coup
--
-- Copyright (c) 2014-2016, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

local TilelayerRenderer = {}
TilelayerRenderer.__index = TilelayerRenderer

function TilelayerRenderer:getBlock(x, y)
  local bs = self.blockSize
  local map = self.layer.map
  local tw, th = map.tilewidth, map.tileheight
  local bw, bh = bs * tw, bs * th
  return math.floor(x / bw), math.floor(y / bh)
end


function TilelayerRenderer:update(x, y, img, q)

end


function TilelayerRenderer:prepare()
  local l = self.layer
  local m = l.map
  local w, h = m.width, m.height
  local tw, th = m.tilewidth, m.tileheight
  local b = self.blocks
  local bs = self.blockSize

  local xb2, yb2 = self:getBlock(w*tw, h*th)

  for y = 0, yb2 do
    by = {}
    b[y] = by
    for x = 0, yb2 do
      byx = {}
      sbs = {}
      by[x] = byx
      local bx, by = self:getBlock(x, y)

      l:foreachInRange(x*bs, y*bs, (x+1)*bs-1, (y+1)*bs-1, function(img, x, y, q)
        local sb = sbs[img]
        if not sb then
          sb = {}
          sbs[img] = sb
        end

        sb[#sb+1] = {x, y, q}
      end)

      for k, v in pairs(sbs) do
        local newb = love.graphics.newSpriteBatch(k, #v)
        byx[k] = newb

        for i, q in ipairs(v) do
          newb:add(q[3], q[1] * tw, q[2] * th)
        end
      end
    end
  end
end


function TilelayerRenderer:draw(topLeft, bottomRight)
  
  local x1, y1 = topLeft.x, topLeft.y
  local x2, y2 = bottomRight.x, bottomRight.y

  local xb1, yb1 = self:getBlock(x1, y1)
  local xb2, yb2 = self:getBlock(x2, y2)

  love.graphics.push()

  local cam_size, screen_offset, offset = coup.camera.cameraSettings(coup.vector.Vector(x1, y1), 
                                                                     coup.vector.Vector(x2, y2))

  love.graphics.translate(math.floor(offset.x), math.floor(offset.y))

  for yb = yb1, yb2 do
    local b = self.blocks[yb]
    if b then
      for xb = xb1, xb2 do
        local bb = b[xb] 
        if bb then
          for k, v in pairs(bb) do
            love.graphics.draw(v)
          end
        end
      end
    end
  end

  love.graphics.pop()
end


local function newTilelayerRenderer(tilelayer, blockSize)
  local r = setmetatable({
    layer = tilelayer,
    blocks = {},
    blockSize = blockSize or 32
  }, TilelayerRenderer)

  r:prepare()

  tilelayer:on_update(function(x, y, img, q)
    r:update(x, y, img, q)
  end)

  return r
end


return { TilelayerRenderer = newTilelayerRenderer }
