--
-- coup
--
-- Copyright (c) 2014-2016, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

local Tileset = {}
Tileset.__index = Tileset

local function makeQuads(image, tileWidth, tileHeight)
  local quads = {}
  local iw, ih = image:getDimensions()

  local tilesPerCol = math.floor(ih / tileHeight) 
  local tilesPerRow = math.floor(iw / tileWidth)

  local mkQuad = coup.cache(love.graphics.newQuad)

  for r = 0, tilesPerCol do
    for c = 0, tilesPerRow do
      quads[r*tilesPerRow+c+1] = mkQuad(c * tileWidth, r * tileHeight, tileWidth, tileHeight, iw, ih)
    end
  end

  return quads
end

function Tileset:get(idx)
  return self.tiles[idx+1-self.first]
end

local function newTileset(image, first, tileWidth, tileHeight)
  local quads = makeQuads(image, tileWidth, tileHeight)
  return setmetatable({
    first = first,
    image = image,
    tileWidth = tileWidth,
    tileHeight = tileHeight,
    tiles = quads,
    tileCount = #quads
  }, Tileset)
end


return { Tileset = newTileset }
