--
-- coup
--
-- Copyright (c) 2014-2016, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

local BASE = (...):match("(.-)[^%.]+$")
local tables = require(BASE .. ".tables")

local TimeKeeper = {
  update = function(self, dt)
    -- update time
    self.currentTime = self.currentTime + dt

    -- run immediate triggers
    for i,v in ipairs(self.immediateTriggers) do
      v(dt)
      self.immediateTriggers[i] = nil
    end

    -- run appropriate timed triggers
    while #self.triggers > 0 and self.triggers[1].time <= self.currentTime do
      local f = self.triggers[1].func
      local d, a = f(unpack(self.triggers[1].args))

      if d then
        if not a then
          a = self.triggers[1].args
        end
        if type(d) ~= 'number' then
           d = self.triggers[1].period
        end
        self:after(d, self.triggers[1].func, unpack(a))
      end
      table.remove(self.triggers, 1)
    end
  end,

  after = function(self, seconds, func, ...)
    if seconds <= 0 then
      table.insert(self.immediateTriggers, {func = func, args = {...}})
      return
    end

    local triggerTime = self.currentTime + seconds

    local index = tables.first(function(t) return t.time >= triggerTime end, self.triggers)
    if not index then
      index = #self.triggers  + 1
    end
    table.insert(self.triggers, index, {time = triggerTime, func =  func, period = seconds, args = {...}})
  end
}

TimeKeeper.__index = TimeKeeper

return function()
  return setmetatable({
    currentTime       = 0,
    triggers          = {},
    immediateTriggers = {}
  }, TimeKeeper)
end
