--
-- coup
--
-- Copyright (c) 2014-2016, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--


coup = require "coup"

function love.load()
  G = {}
  G.world = coup.world.World()

  coup.components.register(G.world)

  G.spriteSpatialHash     = coup.spatialhash.SpatialHashtable(50)
  G.spriteAutoHasher      = coup.systems.AutoHasher(G.spriteSpatialHash, G.world, "Sprite")
  G.aabbUpdater           = coup.systems.AABBUpdater(G.world)
  G.spriteRenderer        = coup.systems.SpriteRenderer(G.spriteSpatialHash)
  G.spriteUpdater         = coup.systems.SpriteUpdater(G.world)
  G.velocityUpdater       = coup.systems.VelocityUpdater(G.world)

  G.animRepo              = coup.sprite.AnimationRepository()
  G.imageCache            = coup.cache(love.graphics.newImage)

  -- Example stuff
  coup.spriteloader.loadAll("data/sprites", G.animRepo, G.imageCache)

  G.tilemap = coup.tiledloader.loadTiledMapFile("data/tilemaps/untitled.lua", G.imageCache)

  G.tileLayerRenderer = coup.tilelayerrenderer.TilelayerRenderer(G.tilemap:get_layer("Tile Layer 1"), 4)

  for i = 1, 1000 do
    local e = G.world:newEntity("e" .. i)
    e:makeComponent("Transform",   math.random(0, 800), math.random(1, 600))
    e:makeComponent("Sprite",      G.animRepo:newSprite("Anim" .. tostring(math.random(2,4))))
    e:makeComponent("BoundingBox", -10, -10, 10, 10)
    e:makeComponent("Velocity",    math.random(-100, 100), math.random(-100, 100))
  end

  G.searchPos = {0 ,0}

  G.camera = G.world:newEntity("camera")
  G.camera:makeComponent("Transform", 400, 300)
  G.camera:makeComponent("BoundingBox", -400, -100, 400, 100)
  
  G.world:step()  -- make sure all entities created in load() are alive

end


function love.update(dt)
  G.velocityUpdater:update(dt)
  G.aabbUpdater:update()
  G.spriteAutoHasher:update()
  G.world:step()
  G.spriteUpdater:updateSprites(dt)
end


function love.draw()
  love.graphics.clear()

  local cab = G.camera:getComponent("AxisAlignedBoundingBox")
  G.tileLayerRenderer:draw(cab.topLeft, cab.bottomRight)
  G.spriteRenderer:renderVisible(G.camera)
end


function love.keypressed(key, scancode, isrepeat)
end


function love.keyreleased(key, scancode)

end


function love.mousemoved(x, y, dx, dy, istouch)
end


function love.mousepressed(x, y, button, istouch)

end


function love.mousereleased(x, y, button, istouch)

end
