--
-- coup
--
-- Copyright (c) 2014-2016, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

package.path = "../coup/?.lua;?.lua"
local cache = require "cache"
local test = require "test"


test.group("cache")

:case("no parameters, empty result", function()
  local count = 0
  local f = function() count = count + 1 end
  local cf = cache(f)

  assert(select('#', cf()) == 0)
  assert(count == 1)
  cf()
  assert(count == 1)
  cf()
  assert(count == 1)
  cf()
  assert(count == 1)
end)


:case("no parameters, single result", function()
  local first = true
  local f = function()
    if first then
      first = false
      return 1 
    else
      return 2
    end
  end
  local cf = cache(f)
  assert(select('#', cf()) == 1)
  assert(cf() == 1)
  assert(cf() == 1)
  assert(cf() == 1)
  assert(cf() == 1)
end)


:case("no parameters, multiple results", function()
  local first = true
  local f = function()
    return 1, 2, 3, 4, {}
  end
  local cf = cache(f)
  assert(select('#', cf()) == 5)
  assert(select(1, cf()) == 1)
  assert(select(2, cf()) == 2)
  assert(select(3, cf()) == 3)
  assert(select(4, cf()) == 4)
  local r5 = select(5, cf())
  assert(type(r5) == 'table')
  assert(#r5 == 0)
end)


:case("one parameter", function()
  local count = 0
  local f = function(x)
    count = count + 1
    return 2*x
  end

  local cf = cache(f)

  assert(cf(1) == 2)
  assert(count == 1)
  assert(cf(1) == 2)
  assert(count == 1)
  assert(cf(1) == 2)
  assert(count == 1)
  assert(cf(1) == 2)
  assert(count == 1)


  assert(cf(2) == 4)
  assert(count == 2)
  assert(cf(2) == 4)
  assert(count == 2)
  assert(cf(2) == 4)
  assert(count == 2)
  assert(cf(2) == 4)
  assert(count == 2)


  assert(cf(3) == 6)
  assert(count == 3)
  assert(cf(3) == 6)
  assert(count == 3)
  assert(cf(3) == 6)
  assert(count == 3)
  assert(cf(3) == 6)
  assert(count == 3)


  assert(cf(1) == 2)
  assert(count == 3)
  assert(cf(1) == 2)
  assert(count == 3)
  assert(cf(1) == 2)
  assert(count == 3)
  assert(cf(1) == 2)
  assert(count == 3)
end)


:case("multiple parameters", function()
  local count = 0
  local f = function(n, ...)
    count = count + 1
    return select(n, ...)
  end

  local cf = cache(f)

  assert(cf(1, 2, 3, 4) == 2)
  assert(count == 1)
  assert(cf(1, 2, 3, 4) == 2)
  assert(count == 1)
  assert(cf(1, 2, 3, 4) == 2)
  assert(count == 1)
  assert(cf(1, 2, 3, 4) == 2)
  assert(count == 1)
  assert(cf(1, 2, 3, 4) == 2)
  assert(count == 1)

  assert(cf(2, 2, 3, 4) == 3)
  assert(count == 2)
  assert(cf(2, 2, 3, 4) == 3)
  assert(count == 2)
  assert(cf(2, 2, 3, 4) == 3)
  assert(count == 2)
  assert(cf(2, 2, 3, 4) == 3)
  assert(count == 2)
  assert(cf(2, 2, 3, 4) == 3)
  assert(count == 2)
  

  assert(cf(3, 2, 3, 4) == 4)
  assert(count == 3)
  assert(cf(3, 2, 3, 4) == 4)
  assert(count == 3)
  assert(cf(3, 2, 3, 4) == 4)
  assert(count == 3)
  assert(cf(3, 2, 3, 4) == 4)
  assert(count == 3)
  assert(cf(3, 2, 3, 4) == 4)
  assert(count == 3)

  assert(cf(1, 2, 3, 4) == 2)
  assert(count == 3)
  assert(cf(1, 2, 3, 4) == 2)
  assert(count == 3)
  assert(cf(1, 2, 3, 4) == 2)
  assert(count == 3)
  assert(cf(1, 2, 3, 4) == 2)
  assert(count == 3)
  assert(cf(1, 2, 3, 4) == 2)
  assert(count == 3)
end)
