--
-- coup
--
-- Copyright (c) 2014-2016, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

package.path = "../coup/?.lua;?.lua"
local class = require "class"
local test = require "test"


test.group("class")

:case("Simple Objects", function()
  local constructorRun = false
  local TestClass = class {
    new = function(self)
      self.x = 1
      constructorRun = true
    end,

    setX = function(self, x)
      self.x = x
    end,

    getX = function(self)
      return self.x
    end
  }

  local testInstance = TestClass()
  assert(constructorRun,                  "Constructor did not run")
  assert(testInstance,                    "Instance not returned")
  assert(testInstance.class == TestClass, "instance.class not set correctly")
  assert(testInstance.x == 1,             "Attribute not set correctly")
  testInstance:setX(5)
  assert(testInstance:getX() == 5,        "Getter or setter not working")
end)


:case("Inheritance", function()
  local BaseClass = class {
    new = function(self, x)
      self.x = x
    end,

    test = function()
      return "base"
    end
  }

  local SubClass = class(BaseClass, {
    new = function(self, x, y)
      self.super.new(self, x)
      self.y = y
    end,

    test = function()
      return "sub"
    end,

    test2 = function(self)
      return self.super.test(self)
    end
  })

  local s = SubClass(10, 20)
  assert(s.x == 10,                    "Base class constructor did not run correctly")
  assert(s.y == 20,                    "Subclass constructor did not run correctly")
  assert(s.test() == "sub",            "Method not correctly overridden")
  assert(s.class == SubClass,          "Wrong class set in instance")
  assert(SubClass:inherits(BaseClass), "Inheritance not set correctly")
  assert(s.super == BaseClass,         "Superclass not available from derived class")
  assert(s:test2() == "base",          "Access of superclass methods not working correctly")
end)
