--
-- coup
--
-- Copyright (c) 2014-2016, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

package.path = "../coup/?.lua;?.lua"

local test = require "test"
local ComponentStore = require "components.componentstore"
ctables = require "tables"

test.group("ComponentStore")

:case("all functionality", function()
  local a, b, c
  local function make(x, y, z)
    a, b, c = x, y, z
    return {x = x, y = y, z = z}
  end

  local store = ComponentStore(make)

  assert(store.count == 0)
  local comp = store:make(1,2,3)

  assert(comp.x == 1)
  assert(comp.y == 2)
  assert(comp.z == 3)
  assert(a == 1)
  assert(b == 2)
  assert(c == 3)

  assert(store.count == 1)
  local comp2 = store:make(1,2,3)

  assert(comp2 ~= comp)
  assert(store.count == 2)

  assert(store.components[comp] == comp)
  assert(store.components[comp2] == comp2)

  store:remove(comp)

  assert(store.count == 1)
  assert(store.components[comp] == nil)
  assert(store.components[comp2] == comp2)

  test.assert_error(function() store:remove(comp) end)

  assert(store.count == 1)
  assert(store.components[comp2] == comp2)

  store:remove(comp2)

  assert(store.count == 0)
  assert(store.components[comp] == nil)
  assert(store.components[comp2] == nil)

end)


:case("onMake, onRemove and onAttach", function()
  local function make() return {} end

  local m = {}
  local r = {}
  local m2 = {}
  local r2 = {}
  local a = {}
  local a2 = {}

  function onMake(comp)
    m[#m+1] = comp
  end

  function onRemove(comp)
    r[#r+1] = comp
  end

  function onMake2(comp)
    m2[#m2+1] = comp
  end

  function onRemove2(comp)
    r2[#r2+1] = comp
  end

  function onAttach(comp)
    a[#a+1] = comp
  end

  function onAttach2(comp)
    a2[#a2+1] = comp
  end


  local store = ComponentStore(make)
  store:onMake(onMake)
  store:onRemove(onRemove)
  store:onAttach(onAttach)

  assert(#m == 0)
  assert(#r == 0)
  assert(#a == 0)

  local e = store:make()

  store:onMake(onMake2)

  assert(#m == 1)
  assert(m[1] == e)
  assert(#r == 0)
  assert(#a == 0)
  
  store:attached(e)

  assert(#a == 1)
  assert(a[1] == e)
  assert(#a2 == 0)

  store:onAttach(onAttach2)
  a = {}

  local e2 = store:make()

  assert(#m2 == 1)
  assert(m2[1] == e2)
  assert(#m == 2)
  assert(m[1] == e)
  assert(m[2] == e2)
  assert(#r == 0)

  store:attached(e2)
  assert(#a == 1)
  assert(a[1] == e2)
  assert(#a2 == 1)
  assert(a2[1] == e2)

  m = {}
  store:remove(e)

  assert(#r == 1)
  assert(r[1] == e)
  assert(#m == 0)

  store:onRemove(onRemove2)

  store:remove(e2)
  assert(#r2 == 1)
  assert(r2[1] == e2)
  assert(#r == 2)
  assert(r[1] == e)
  assert(r[2] == e2)
  assert(#m == 0)

  m = {}
  r = {}
  m2 = {}
  r2 = {}
  a = {}
  a2 = {}

  store:removeOnRemove(onRemove)
  store:removeOnMake(onRemove)
  store:removeOnAttach(onAttach)

  store:removeOnRemove(onMake) -- must not fail or remove anything else
  store:removeOnMake(onMake) -- must not fail or remove anything else

  e = store:make()
  assert(#m == 0)
  assert(#m2 == 1)

  store:attached(e)
  assert(#m == 0)
  assert(#m2 == 1)
  assert(m2[1] == e)

  store:remove(e)
  assert(#r == 0)
  assert(#r2 == 1)




end)
