--
-- coup
--
-- Copyright (c) 2014-2016, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

package.path = "../coup/?.lua;?.lua"
local entity = require "entity"
local ComponentStore = require "components.componentstore"
local test = require "test"


test.group("Entity")

:case("newEntity", function()
  local world = {}
  local e = entity.Entity(world, "Test")

  assert(e.name == "Test")
  assert(e.world == world)
end)


:case("addComponent/getComponent", function()
  local e = entity.Entity("Test")
  test.assert_error(e.addComponent, e, {})

  local function make() return {type="Foo"} end

  local store = ComponentStore(make)

  local c = store:make()
  e:addComponent(c)
  test.assert_error(e.addComponent, e, {type = "Foo"})

  local e2 = entity.Entity("Test2")
  test.assert_error(e2.addComponent, e, c)

  assert(e:getComponent("Foo") == c)
  assert(e2:getComponent("Foo") == nil)
  assert(c.entity == e)
end)


:case("removeComponent/addComponent: removeComponent event", function()
  local e = entity.Entity("Test")

  local function make()
    return {type="Foo"}
  end

  local cs = ComponentStore(make)
  cs.remove = function(self, comp)
    self.re = comp
  end

  local c = cs:make()

  e:addComponent(c)
  e:removeComponent(c)
  assert(cs.re == c)

  cs.re = nil
  e:addComponent(c)
  e:removeComponent("Foo")
  assert(cs.re == c)

end)


:case("destroy", function()
  local e = entity.Entity("Test")
  
  local removed = {}
  local function make(t) return {type=t} end
  local cs = ComponentStore(make)

  cs:onRemove(function(c) removed[c] = c end)

  local c1 = cs:make("Foo")
  local c2 = cs:make("Bar")

  e:addComponent(c1)
  e:addComponent(c2)
  e:destroy()

  assert(e:getComponent("Foo") == nil)
  assert(e:getComponent("Bar") == nil)

  assert(removed[c1] == c1)
  assert(removed[c2] == c2)
end)
