--
-- coup
--
-- Copyright (c) 2014-2016, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

package.path = "../coup/?.lua;?.lua"
local smath = require "smath"
local test = require "test"

test.group("scalar maths")

:case("Square", function()
  assert(smath.sq(0) == 0)
  assert(smath.sq(1) == 1)
  assert(smath.sq(-1) == 1)
  assert(smath.sq(2) == 4)
end)


:case("Cube", function()
  assert(smath.cu(0) == 0)
  assert(smath.cu(1) == 1)
  assert(smath.cu(-1) == -1)
  assert(smath.cu(2) == 8)
end)


:case("Lerp", function()
  for i = 0, 1, 0.1 do
    assert(smath.lerp(0, 1, i) == i)
  end

  assert(smath.lerp(-5, 5, 0.5) == 0)
  assert(smath.lerp(-10, 0, 0) == -10)
  assert(smath.lerp(0, 20, 0.75) == 15)
end)


:case("Interpolate - linear", function()
  for i = 0, 1, 0.1 do
    assert(smath.interpolate(0, 1, i, "linear") == i)
  end

  assert(smath.interpolate(-5, 5, 0.5, "linear") == 0)
  assert(smath.interpolate(-10, 0, 0, "linear") == -10)
  assert(smath.interpolate(0, 20, 0.75, "linear") == 15)
end)


:case("Interpolate - sine_inout", function()
  local points = {
    {0.0, 10},
    {0.1, 10.244717418524},
    {0.2, 10.954915028125},
    {0.3, 12.061073738538},
    {0.4, 13.454915028125},
    {0.5, 15},
    {0.6, 16.545084971875},
    {0.7, 17.938926261462},
    {0.8, 19.045084971875},
    {0.9, 19.755282581476},
    {1.0, 20}
  }

  for _, p in ipairs(points) do
    assert(math.abs(smath.interpolate(10,20,p[1],"sine_inout") - p[2]) < 0.000001)
  end
end)


:case("Interpolate - sine_in", function()
  local points = {
    {0.0, 10},
    {0.1, 10.123116594049},
    {0.2, 10.489434837048},
    {0.3, 11.089934758116},
    {0.4, 11.909830056251},
    {0.5, 12.928932188135},
    {0.6, 14.122147477075},
    {0.7, 15.460095002605},
    {0.8, 16.909830056251},
    {0.9, 18.435655349598},
    {1.0, 20},
  }

  for _, p in ipairs(points) do
    assert(math.abs(smath.interpolate(10,20,p[1],"sine_in") - p[2]) < 0.000001)
  end
end)


:case("Interpolate - sine_out", function()
  local points = {
    {0.0, 10},
    {0.1, 11.564344650402},
    {0.2, 13.090169943749},
    {0.3, 14.539904997395},
    {0.4, 15.877852522925},
    {0.5, 17.071067811865},
    {0.6, 18.090169943749},
    {0.7, 18.910065241884},
    {0.8, 19.510565162952},
    {0.9, 19.876883405951},
    {1.0, 20},
  }

  for _, p in ipairs(points) do
    assert(math.abs(smath.interpolate(10,20,p[1],"sine_out") - p[2]) < 0.000001)
  end
end)


:case("interpolate_func", function()

  for i, m in ipairs({"sine_out", "sine_in", "sine_inout", "linear"}) do
    local fn = smath.interpolate_func(m)

    for i = 0, 1, 0.1 do
      assert(fn(0, 100, i) == smath.interpolate(0, 100, i, m))
    end
  end

end)


:case("between", function()
  assert(not smath.between(0,   1, 2))
  assert(    smath.between(1,   1, 2))
  assert(    smath.between(1.5, 1, 2))
  assert(    smath.between(2,   1, 2))
  assert(not smath.between(3,   1, 2))

  -- Reverse bounds
  assert(not smath.between(0,   2, 1))
  assert(    smath.between(1,   2, 1))
  assert(    smath.between(1.5, 2, 1))
  assert(    smath.between(2,   2, 1))
  assert(not smath.between(3,   2, 1))
end)


:case("sign", function()
  assert(smath.sign(-1) == -1)
  assert(smath.sign( 1) ==  1)
  assert(smath.sign(-0) ==  0)
  assert(smath.sign( 0) ==  0)
end)
