--
-- coup
--
-- Copyright (c) 2014-2016, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

package.path = "../coup/?.lua;?.lua"
local sprite = require "sprite"
local test = require "test"

test.group("sprite")

:case("animation repository", function()
  local repo = sprite.AnimationRepository()
  local anim = repo:addAnimation("test")
  local test = repo:addAnimation("test2")

  local test_img = {getDimensions = function() return 64, 128 end}
  local test_img2 = {getDimensions = function() return 32, 32 end}

  anim:addFrame(test_img,   0,  0, 32, 32, 16, 16, 0.125)
  anim:addFrame(test_img,  40,  8, 16, 16,  8,  8, 0.25)
  anim:addFrame(test_img,   0, 32, 32, 32, 16, 16, 0.125)
  anim:addFrame(test_img2, 16, 16, 16, 16,  2,  4, 0.0625)
  assert(anim:frameCount() == 4)

  test:addFrame(test_img, 0, 0, 4, 4, 2, 2, 1.0)
  assert(test:frameCount() == 1)

  local frame = test:getFrame(0)
  assert(frame.img == test_img)
  assert(frame.start == 0.0)
  assert(frame.delay == 1.0)
  assert(frame.origin.x == 2)
  assert(frame.origin.y == 2)
  
  local x, y, w, h = frame.quad:getViewport()
  assert(x == 0)
  assert(y == 0)
  assert(w == 4)
  assert(h == 4)

  local iw, ih = frame.quad:getTextureDimensions()
  assert(iw == 64)
  assert(ih == 128)


  local frame0 = anim:getFrame(0)
  assert(frame0.img == test_img)
  assert(frame0.start == 0.0)
  assert(frame0.delay == 0.125)
  assert(frame0.origin.x == 16)
  assert(frame0.origin.y == 16)
  
  x, y, w, h = frame0.quad:getViewport()
  assert(x == 0)
  assert(y == 0)
  assert(w == 32)
  assert(h == 32)

  iw, ih = frame0.quad:getTextureDimensions()
  assert(iw == 64)
  assert(ih == 128)


  local frame1 = anim:getFrame(1)
  assert(frame1.img == test_img)
  assert(frame1.start == 0.125)
  assert(frame1.delay == 0.25)
  assert(frame1.origin.x == 8)
  assert(frame1.origin.y == 8)
  
  x, y, w, h = frame1.quad:getViewport()
  assert(x == 40)
  assert(y == 8)
  assert(w == 16)
  assert(h == 16)

  iw, ih = frame1.quad:getTextureDimensions()
  assert(iw == 64)
  assert(ih == 128)


  local frame2 = anim:getFrame(2)
  assert(frame2.img == test_img)
  assert(frame2.start == 0.375)
  assert(frame2.delay == 0.125)
  assert(frame2.origin.x == 16)
  assert(frame2.origin.y == 16)
  
  x, y, w, h = frame2.quad:getViewport()
  assert(x ==  0)
  assert(y == 32)
  assert(w == 32)
  assert(h == 32)

  iw, ih = frame2.quad:getTextureDimensions()
  assert(iw == 64)
  assert(ih == 128)


  local frame3 = anim:getFrame(3)
  assert(frame3.img == test_img2)
  assert(frame3.start == 0.5)
  assert(frame3.delay == 0.0625)
  assert(frame3.origin.x == 2)
  assert(frame3.origin.y == 4)
  
  x, y, w, h = frame3.quad:getViewport()
  assert(x ==  16)
  assert(y == 16)
  assert(w == 16)
  assert(h == 16)

  iw, ih = frame3.quad:getTextureDimensions()
  assert(iw == 32)
  assert(ih == 32)

  local anim2 = repo:getAnimation("test")
  assert(anim == anim2)

  local test2 = repo:getAnimation("test2")
  assert(test2 == test)

  assert(nil == anim:getFrameForTime(-0.00001))
  assert(0 == anim:getFrameForTime(0.00))
  assert(0 == anim:getFrameForTime(0.05))
  assert(0 == anim:getFrameForTime(0.09))
  assert(0 == anim:getFrameForTime(0.09))
  assert(1 == anim:getFrameForTime(0.125))
  assert(1 == anim:getFrameForTime(0.13))
  assert(1 == anim:getFrameForTime(0.375-0.00001))
  assert(2 == anim:getFrameForTime(0.375))
  assert(nil == anim:getFrameForTime(0.6))
end)


:case("sprites basics", function()
  local repo = sprite.AnimationRepository()
  local anim = repo:addAnimation("test")

  local test_img = {getDimensions = function() return 64, 64 end}
  anim:addFrame(test_img,  0,  0, 32, 32, 16, 16, 0.125)
  anim:addFrame(test_img, 32,  0, 32, 32, 16, 16, 0.25)
  anim:addFrame(test_img,  0, 32, 32, 32, 16, 16, 0.125)
  anim:addFrame(test_img, 32, 32, 32, 32, 16, 16, 0.5)

  local sprite = repo:newSprite()

  assert(sprite:getRepository() == repo)
  assert(sprite:getCurrentFrame() == nil)

  sprite:play("test")
  assert(sprite:getCurrentFrame() == anim:getFrame(0))

  sprite:update(0.05)
  assert(sprite:getCurrentFrame() == anim:getFrame(0))

  sprite:update(0.05)
  assert(sprite:getCurrentFrame() == anim:getFrame(0))

  sprite:update(0.02499999)
  assert(sprite:getCurrentFrame() == anim:getFrame(0))

  sprite:update(0.00000001)
  assert(sprite:getCurrentFrame() == anim:getFrame(1))

  sprite:update(0.4)
  assert(sprite:getCurrentFrame() == anim:getFrame(3))

  sprite:update(0.5)
  assert(sprite:getCurrentFrame() == anim:getFrame(0))

  sprite:update(1.0)
  assert(sprite:getCurrentFrame() == anim:getFrame(0))

  sprite:update(20000.0)
  assert(sprite:getCurrentFrame() == anim:getFrame(0))

  sprite:update(0.025)
  assert(sprite:getCurrentFrame() == anim:getFrame(0))

  sprite:setSpeed(2)
  sprite:play("test")
  
  sprite:update(0.1)
  assert(sprite:getCurrentFrame() == anim:getFrame(1))
  sprite:update(0.5)
  assert(sprite:getCurrentFrame() == anim:getFrame(1))
  sprite:update(0.0874)
  assert(sprite:getCurrentFrame() == anim:getFrame(1))
  sprite:update(0.001)
  assert(sprite:getCurrentFrame() == anim:getFrame(2))

  sprite:play("test", true)
  assert(sprite:getCurrentFrame() == anim:getFrame(2))

  sprite:play("test")
  assert(sprite:getCurrentFrame() == anim:getFrame(0))
end)


:case("sprites playback speed", function()
  local repo = sprite.AnimationRepository()
  local anim = repo:addAnimation("test")

  local test_img = {getDimensions = function() return 64, 64 end}
  anim:addFrame(test_img,  0,  0, 32, 32, 16, 16, 0.125)
  anim:addFrame(test_img, 32,  0, 32, 32, 16, 16, 0.25)
  anim:addFrame(test_img,  0, 32, 32, 32, 16, 16, 0.125)
  anim:addFrame(test_img, 32, 32, 32, 32, 16, 16, 0.5)

  local sprite = repo:newSprite()

  sprite:setSpeed(2)
  sprite:play("test")
  
  sprite:update(0.1)
  assert(sprite:getCurrentFrame() == anim:getFrame(1))
  sprite:update(0.5)
  assert(sprite:getCurrentFrame() == anim:getFrame(1))
  sprite:update(0.0874)
  assert(sprite:getCurrentFrame() == anim:getFrame(1))
  sprite:update(0.001)
  assert(sprite:getCurrentFrame() == anim:getFrame(2))
end)


:case("sprites animation chaining", function()
  local repo = sprite.AnimationRepository()
  local test_img = {getDimensions = function() return 64, 64 end}
  local anim = repo:addAnimation("test")
  anim:addFrame(test_img,  0,  0, 32, 32, 16, 16, 0.25)
  anim:addFrame(test_img, 32,  0, 32, 32, 16, 16, 0.25)
  anim:addFrame(test_img,  0, 32, 32, 32, 16, 16, 0.25)
  anim:addFrame(test_img, 32, 32, 32, 32, 16, 16, 0.25)

  local anim2 = repo:addAnimation("test2")
  anim2:addFrame(test_img,  0,  0, 32, 32, 16, 16, 0.25)
  anim2:addFrame(test_img, 32,  0, 32, 32, 16, 16, 0.25)
  anim2:addFrame(test_img,  0, 32, 32, 32, 16, 16, 0.25)
  anim2:addFrame(test_img, 32, 32, 32, 32, 16, 16, 0.25)

  local sprite = repo:newSprite()

  sprite:play("test")
  sprite:chain("test2")
  
  sprite:update(0.3)
  assert(sprite:getCurrentFrame() == anim:getFrame(1))
  sprite:update(0.3)
  assert(sprite:getCurrentFrame() == anim:getFrame(2))
  sprite:update(0.3)
  assert(sprite:getCurrentFrame() == anim:getFrame(3))

  -- roll over to next anim
  sprite:update(0.3)
  assert(sprite:getCurrentFrame() == anim2:getFrame(0))

  -- is the time overflow carried over to the next anim?
  assert(math.abs(sprite.time - 0.2) < 0.00001)
end)


:case("sprites animation end events", function()
  local repo = sprite.AnimationRepository()
  local test_img = {getDimensions = function() return 64, 64 end}
  local anim = repo:addAnimation("test")
  anim:addFrame(test_img,  0,  0, 32, 32, 16, 16, 0.25)
  anim:addFrame(test_img, 32,  0, 32, 32, 16, 16, 0.25)
  anim:addFrame(test_img,  0, 32, 32, 32, 16, 16, 0.25)
  anim:addFrame(test_img, 32, 32, 32, 32, 16, 16, 0.25)

  local sprite = repo:newSprite()
  local otherSprite = repo:newSprite()

  sprite:play("test")
  local endEventCounter = 0
  local eventSprite
  sprite:onEnd(function(spr)
    endEventCounter = endEventCounter + 1
    eventSprite = spr
  end)
  
  sprite:update(0.5)
  assert(endEventCounter == 0)
  sprite:update(0.6)
  assert(endEventCounter == 1)
  assert(eventSprite == sprite)
  sprite:update(0.5)
  sprite:update(1.0)
  assert(endEventCounter == 1)

  local f
  f = function(spr)
    endEventCounter = endEventCounter + 1
    if endEventCounter < 5 then
      spr:onEnd(f)
    end
  end
  sprite:onEnd(f)
  sprite:update(1)
  sprite:update(1)
  sprite:update(1)
  sprite:update(1)
  assert(endEventCounter == 5)
  sprite:update(1)
  assert(endEventCounter == 5)
  endEventCounter = 0
  sprite:onEnd(f)
  sprite:update(4)
  assert(endEventCounter == 4)
  sprite:update(4)
  assert(endEventCounter == 5)

  local bFired = 0
  local f1 = function(spr)
    bFired = bFired + 1
  end

  local f2 = function(spr)
    spr:onEnd(f1)
  end

  sprite:play("test")
  sprite:onEnd(f2)
  sprite:update(1000)
  assert(bFired == 1)


  sprite:play("test")
  sprite:onEnd(f1, true)
  bFired = 0
  sprite:update(1000)
  assert(bFired == 1000)
end)

