--
-- coup
--
-- Copyright (c) 2014-2016, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

package.path = "../coup/?.lua;?.lua"
local timekeeper = require "timekeeper"
local test = require "test"

test.group("Timekeeper")

:case("Single trigger without arguments", function()
  local tk = timekeeper()

  local done = false
  tk:after(2, function()
    done = true
  end)

  tk:update(1)
  assert(not done)

  for i = 1, 9 do
    tk:update(0.1)
    assert(not done)
  end

  tk:update(0.2)
  assert(done)
end)


:case("Single trigger with arguments", function()
  local tk = timekeeper()

  local done = 0
  tk:after(2, function(x)
    done = x
  end, 17)

  tk:update(1)
  assert(done == 0)

  for i = 1, 9 do
    tk:update(0.1)
    assert(done == 0)
  end

  tk:update(0.2)
  assert(done == 17)
end)


:case("Multiple triggers", function()
  local tk = timekeeper()

  local done = {0, 0, 0, 0}
  
  for i = 1, 4 do
    tk:after(i, function(x) done[i] = i end, i)
  end

  tk:update(0.5)
  assert(done[1] == 0)
  assert(done[2] == 0)
  assert(done[3] == 0)
  assert(done[4] == 0)

  tk:update(0.5)
  assert(done[1] == 1)
  assert(done[2] == 0)
  assert(done[3] == 0)
  assert(done[4] == 0)

  tk:update(0.5)
  assert(done[1] == 1)
  assert(done[2] == 0)
  assert(done[3] == 0)
  assert(done[4] == 0)

  tk:update(0.5)
  assert(done[1] == 1)
  assert(done[2] == 2)
  assert(done[3] == 0)
  assert(done[4] == 0)

  tk:update(0.9)
  assert(done[1] == 1)
  assert(done[2] == 2)
  assert(done[3] == 0)
  assert(done[4] == 0)

  tk:update(0.2)
  assert(done[1] == 1)
  assert(done[2] == 2)
  assert(done[3] == 3)
  assert(done[4] == 0)

  tk:update(0.8)
  assert(done[1] == 1)
  assert(done[2] == 2)
  assert(done[3] == 3)
  assert(done[4] == 0)

  tk:update(1)
  assert(done[1] == 1)
  assert(done[2] == 2)
  assert(done[3] == 3)
  assert(done[4] == 4)
end)


:case("Periodic functions", function()
  local tk = timekeeper()

  local c = 0

  tk:after(1, function()
    c = c + 1
    return c < 3
  end)

  tk:update(1)
  assert(c == 1)
  tk:update(0.5)
  assert(c == 1)
  tk:update(0.5)
  assert(c == 2)
  tk:update(0.5)
  assert(c == 2)
  tk:update(5)
  assert(c == 3)
  tk:update(1)
  assert(c == 3)
  tk:update(1)
  assert(c == 3)
  tk:update(1)
  assert(c == 3)
end)
