--
-- coup
--
-- Copyright (c) 2014-2016, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

package.path = "../coup/?.lua;?.lua"
local console  = require "console"

local test = {
  r = 0,
  p = 0,
  f = 0,
  maxTestNameLen = 0,
  groups = {}
}

test.group = function(name)
  local grp = {
    tests = {},
    name = name,
    case = function(self, name, func)
      self.tests[#self.tests+1] = {
        name = name,
        func = func
      }

      test.maxTestNameLen = math.max(test.maxTestNameLen, #name)

      return self
    end,
    run = function(self)
      print("Running group '" .. self.name .. "'...")
      local r, p, f = 0, 0, 0
      for i, t in ipairs(self.tests) do
        io.write(" * Running test '" .. t.name .. "'")
        for i = 1, test.maxTestNameLen - #t.name + 3 do
          io.write('.')
        end
        io.flush()
        local ok, res = pcall(t.func)
        io.write((ok and console.green("pass") or (console.red("fail: ") .. res)) .. "\n")
        io.flush()
        r = r + 1
        p = p + (ok and 1 or 0)
        f = f + (ok and 0 or 1)
      end
      print("Finished group '" .. self.name .. "': " .. console.bright(r) .. " run, " .. console.green(p) .. " passed, " .. console.red(f) .. " failed\n")
      test.r = test.r + r
      test.p = test.p + p
      test.f = test.f + f
    end
  }

  test.groups[#test.groups+1] = grp

  return grp
end


test.assert_error = function(func, ...)
  local err = pcall(func, ...)
  assert(not err)
end


test.run = function()
  for i, g in ipairs(test.groups) do
    g:run()
  end

  print("\n===============================================================================")
  print("All tests run: " .. console.bright(test.r) .. " tests, " .. console.green(test.p) .. " passed, " .. console.red(test.f) .. " failed\n")
end


return test
