
local HighlightRenderer = {}
HighlightRenderer.__index = HighlightRenderer


local function collectSprites(tl, br, hash)
  local h = {}
  for sh in pairs(hash:shapesInRange(tl.x - 100, tl.y - 100, br.x + 100, br.y + 100)) do
    local o = sh.owner
    if not sh.isSensor and o then
      local e = o.entity
      local s, t = e:getComponents('Highlight', 'Transform')
      if s then
        h[#h+1] = {s, t}
      end
    end
  end

  return h
end



function HighlightRenderer:renderVisible(camera, lerp, col, useColor)
  local phys = camera:getComponent("AxisAlignedBoundingBox")

  local layers = collectSprites(phys.topLeft, phys.bottomRight, self.spatialHash)
  --local sortLayers = sortSpriteLayers(layers)

  local cam_size, screen_offset, offset = coup.camera.cameraSettings(phys.topLeft, phys.bottomRight)

  col = col or {255,255,255,255}

  love.graphics.setColor(col)
  local sbs = self.spriteBatch
  sbs:clear()
  sbs:setColor(255,255,255,255)
  for i, d in ipairs(layers) do
    local p = d[2]:lpos(lerp)
    if useColor then
      sbs:setColor(d[1].color)
    end
    sbs:add(math.floor(p.x), math.floor(p.y), 0, self.sclx * d[1].sx, self.scly * d[1].sy, self.ox, self.oy)
  end

  love.graphics.draw(sbs, math.floor(offset.x), math.floor(offset.y))
  sbs:clear()

end


local function newHighlightRenderer(spatialHash)
  local img = G.loadImage("data/highlight.png")
  local sclx, scly = img:getDimensions()
  return setmetatable({
    spatialHash = spatialHash,
    spriteBatch = love.graphics.newSpriteBatch(img, 100),
    sclx = 1 / sclx,
    scly = 1 / scly,
    ox = sclx/2,
    oy = scly/2
  }, HighlightRenderer)
end

return newHighlightRenderer
