PhysicsUpdater = coup.class({})


function PhysicsUpdater.updateEntity(e)
  local v = e:getComponent("Velocity")
  local tr = e:getComponent("Transform")
  local po = e:getComponent("PhysicsObject")

  po.collider:moveTo(tr.pos.x + po.x, tr.pos.y + po.y)

  local sens = e:getComponent("Sensors")
  if sens then
    for k, s in pairs(sens.sensors) do
      s.sensor:moveTo(tr.pos.x+s.x, tr.pos.y+s.y)
    end
  end
end


function PhysicsUpdater:update(dt)

  self.world.world:foreachEntityWithComponents(function(e)
    local v = e:getComponent("Velocity")
    local tr = e:getComponent("Transform")
    local po = e:getComponent("PhysicsObject")

    tr.last_pos = tr.pos:clone()

    if not po.dont then
      v.vel = v.vel + v.acc * dt

      if v.acc.x == 0 then
        -- drag
        local drag = vel.hor_drag
        if v.vel.x < 0 then
          v.vel.x = v.vel.x + drag*dt
          if v.vel.x > 0 then
            v.vel.x = 0
          end
        end

        if v.vel.x > 0 then
          v.vel.x = v.vel.x - drag*dt
          if v.vel.x < 0 then
            v.vel.x = 0
          end
        end
      end
    
      if v.vel.x > v.max_vel.x then
        v.vel.x = v.max_vel.x
      elseif v.vel.x < -v.max_vel.x then
        v.vel.x = -v.max_vel.x
      end

      if v.vel.y > v.max_vel.y then
        v.vel.y = v.max_vel.y
      elseif v.vel.y < -v.max_vel.y then
        v.vel.y = -v.max_vel.y
      end

      v.vel.y = v.vel.y + dt * self.gravity * v.grav
      tr.pos = tr.pos + v.vel * dt
    end

    PhysicsUpdater.updateEntity(e)

  end, "Velocity", "Transform", "PhysicsObject")

  self.collisions = {}
  self.world.collider:update(dt)
end


local function resolveCollision(s, dx, dy)
  if dx == 0 and dy == 0 then
    return
  end

  local c = s.owner
  if not c then
    return
  end

  if c.onGroundCollision then
    c.onGroundCollision()
  end

  local e = c.entity
  if e then
    local v = e:getComponent("Velocity")
    local tr = e:getComponent("Transform")

    tr.pos.x = tr.pos.x + dx
    tr.pos.y = tr.pos.y + dy

    local l = math.sqrt(dx*dx+dy*dy)
    local nx, ny = dx / l, dy / l

    local n_dot_v = nx*v.vel.x + ny*v.vel.y

    local nnx = nx * n_dot_v
    local nny = ny * n_dot_v

    v.vel.x = v.vel.x - nnx
    v.vel.y = v.vel.y - nny

    PhysicsUpdater.updateEntity(e)
  end
end


function PhysicsUpdater:new(world)
  self.gravity = 1000
  self.world = world

  self.world.collider:setCallbacks(function(dt, s1, s2, dx, dy)
    if s1.islevel then
      resolveCollision(s2, -dx, -dy)
    elseif s2.islevel then
      resolveCollision(s1, dx, dy)
    else
      if s1.owner and s2.owner then
        self.collisions[#self.collisions+1] = {dt, s1.owner, s2.owner, dx, dy}
      end
    end
  end)
  self.collisions = {}
end
