--
-- coup
--
-- Copyright (c) 2014-2016, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

local SpriteRenderer = {}
SpriteRenderer.__index = SpriteRenderer


local function collectSprites(tl, br, hash)
  local layers = {}

  for sh in pairs(hash:shapesInRange(tl.x - 100, tl.y - 100, br.x + 100, br.y + 100)) do
    local o = sh.owner
    if not sh.isSensor and o then
      local e = o.entity
      local s, t = e:getComponents('Sprite', 'Transform')
      if s then
        local f = s.sprite:getCurrentFrame()
        if f then
          local layer = layers[s.layer]
          if not layer then
            layer = {}
            layers[s.layer] = layer
          end

          layer[#layer+1] = {f, t, s, e}
        end
      end
    end
  end

  return layers
end


local function compareLayers(a, b)
  return a[1] < b[1] 
end

local function sortSpriteLayers(layers)
  local sortLayers = {}
  for k, v in pairs(layers) do
    sortLayers[#sortLayers+1] = {k, v}
  end

  table.sort(sortLayers, compareLayers)

  return sortLayers
end


local function prepareSpriteBatches(sbs, entries, lerp)
  for k, e in ipairs(entries) do
    local f = e[1]

    local sb = sbs[f.img]
    if sb == nil then
      sb = love.graphics.newSpriteBatch(f.img, 1000, "stream")
      sbs[f.img] = sb
    end
    local t = e[2]
    local sp = e[3]
    --[[
    local p = t.pos
    local lp = t.last_pos
    if not lp then
      lp = p
    end


    local fp = p * lerp + lp * (1-lerp)
    ]]
    local fp = t:lpos(lerp)

    local r = t.rot
    local s = t.scale
    local o = f.origin
    
    sb:add(f.quad, math.floor(fp.x), math.floor(fp.y), r, s.x, s.y, o.x+sp.ox, o.y+sp.oy)
  end
end


function SpriteRenderer:renderVisible(camera, lerp)
  local phys = camera:getComponent("AxisAlignedBoundingBox")

  local layers = collectSprites(phys.topLeft, phys.bottomRight, self.spatialHash)
  local sortLayers = sortSpriteLayers(layers)

  local cam_size, screen_offset, offset = coup.camera.cameraSettings(phys.topLeft, phys.bottomRight)

  love.graphics.setColor(255,255,255,255)
  local sbs = self.spriteBatches
  for i, l in ipairs(sortLayers) do
    prepareSpriteBatches(sbs, l[2], lerp)

    for j, s in pairs(self.spriteBatches) do
      love.graphics.draw(s, math.floor(offset.x), math.floor(offset.y))
      s:clear()
    end
  end

end


local function newSpriteRenderer(spatialHash)
  return setmetatable({
    spatialHash = spatialHash,
    spriteBatches = {}
  }, SpriteRenderer)
end

return newSpriteRenderer
